/*  frontend.c - The frontend to Gqcam, a GTK based QuickPict clone.
    Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)

    This software contains ideas and modified parts from qcread, 
    written by Alex Belits.
    Copyright (C) 1996, 1997 Alex Belits

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */


#include <gtk/gtk.h>
#include "cqc.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

struct qcam *q;
GdkRgbCmap *cmap;

void delete_event( GtkWidget *widget,
		   GdkEvent  *event,
		   gpointer   data )
{
  gtk_main_quit ();
}


gint clear (gpointer data)
{
  GtkWidget* drawing_area = (GtkWidget *) data;
  GdkDrawable *drawable;

  drawable = drawing_area->window;
/*  gdk_draw_rectangle (drawable,
		      drawing_area->style->white_gc,
		      TRUE,
		      0, 0,
		      drawing_area->allocation.width,
		      drawing_area->allocation.height);
  */

  if(qcreadframe(q))
    {
      fputs("gqcam: Frame read failed\n",stderr);
      exit(1);
    }

  gdk_draw_indexed_image (drawable, drawing_area->style->white_gc, 0, 0, q->xsize/q->decim,q->ysize/q->decim, GDK_RGB_DITHER_NONE, q->frame, q->xsize/q->decim, cmap);

/*
  gdk_draw_gray_image (drawable, drawing_area->style->white_gc, 0, 0, q->xsize/q->decim,q->ysize/q->decim, GDK_RGB_DITHER_NONE, q->frame, q->xsize/q->decim);
*/
  return (TRUE);
}

void setbrightness(GtkAdjustment *adj, GtkWidget *none)
{
  int temp;
  temp = adj->value;
  q->brightness = temp;
  qcinitset(q);
}

void setwhite(GtkAdjustment *adj, GtkWidget *none)
{
  int temp;
  temp = adj->value;
  q->blackoffset = temp;
  qcinitset(q);
}

void setcontrast(GtkAdjustment *adj, GtkWidget *none)
{
  int temp;
  temp = adj->value;
  q->contrast = temp;
  qcinitset(q);
}

void setsizefull( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
  q->decim=1;
  qcinitset(q);
}
void setsizehalf( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
  q->decim=2;
  qcinitset(q);
}
void setsizequarter( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
  q->decim=4;
  qcinitset(q);
}

int main( int   argc, char *argv[] )
{
  GtkWidget *window;
  GtkWidget *topbox;
  GtkWidget *outerbox;
  GtkWidget *innerbox;
  GtkWidget *scalebox;
  GtkWidget *sizedepthbox;
  GtkWidget *frame;
  GtkObject *adjb;
  GtkWidget *scaleb;
  GtkObject *adjw;
  GtkWidget *scalew;
  GtkObject *adjc;
  GtkWidget *scalec;
  GtkWidget *radiobutton;
  GSList *radiogroup;
  GtkWidget *button;
  GtkWidget *drawing_area;
  GtkImage *qview;
  GdkDrawable *drawable;
  guchar *gbuff;
  GdkColor c;

  guint32 colors[64];
  gint ncolors=64;
  int i, j;
  int ib=130, ic=30, iw=150;

  gtk_init (&argc, &argv);
  gdk_rgb_init();

  q=qccreate();
  if(!q){
    fputs("gqcam: Insufficient memory\n",stderr);
    exit(1);
  }
  qcreadconfigfile(q,NULL,NULL,0);

  q->brightness=ib;
  q->blackoffset=iw;
/*  q->white=150;*/
  q->decim=1;
  q->contrast=ic;

  for(j=0;j<ncolors;j++) {
    i=j*4;
    colors[j] = i+(i*256)+(i*256*256);

  }
  

  cmap = gdk_rgb_cmap_new(colors, ncolors);


/*  
  cmap = gdk_rgb_cmap_new(colors, ncolors);
*/
  qcinit(q, q->p0);

  switch(qcinitset(q)){
  case -1:
    fputs("gqcam: Initialization failed: Can't run as non-root\n",stderr);
    exit(1);
  case -2:
    perror("gqcam: Initialization failed");
    exit(1);
  case -3:
    fputs("gqcam: Initialization failed: QuickCam not detected\n",stderr);
    exit(1);
  case -4:
    fprintf(stderr,"gqcam: Initialization failed: QuickCam not detected or unsupported type\n");
    exit(1);
  case -5:
    fputs("gqcam: Initialization failed: QuickCam doesn't work\n",stderr);
    exit(1);
  case -6:
    fputs("gqcam: Image setup failed\n",stderr);
    exit(1);
  case -7:
    fputs("gqcam: Invalid frame parameters\n",stderr);
    exit(1);
  case -8:
    fputs("gqcam: Frame setup failed\n",stderr);
    exit(1);
  case -9:
    fputs("gqcam: Frame memory allocation failed\n",stderr);
    exit(1);
  case 0: 
    break;
  default: 
    fprintf(stderr,"gqcam: Unknown error\n");
    exit(1);
  }
  


/*----------------------------------------------------*/
/*
  if(qcreadframe(q)){
    fputs("qcread: Frame read failed\n",stderr);
    exit(1);
  }
  
  printf("P5\n%d %d %d\n",q->xsize/q->decim,q->ysize/q->decim,(q->bpp==4)?15:63);
  
  fflush(stdout);
  write(1,q->frame,q->framesize);
  */


/*------------------------------------------------------*/


/*  
  gtk_init (&argc, &argv);
  
  gdk_rgb_init();
*/
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Gqcam");

  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  
  gtk_container_set_border_width (GTK_CONTAINER (window), 2);

  /* Create topbox */
  topbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), topbox);

  /* Create the drawing area */
  drawing_area = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), 320, 240);
  gtk_box_pack_start (GTK_BOX (topbox), drawing_area, TRUE, TRUE, 0);
  gtk_widget_show (drawing_area);

  /* Create new outerbox */
  outerbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (topbox), outerbox, TRUE, TRUE, 0);

  /* Create scalebox */
  scalebox = gtk_vbox_new (FALSE, 2);
  gtk_widget_set_usize (GTK_WIDGET(scalebox), 240, 150);
    gtk_box_pack_start( GTK_BOX (outerbox), scalebox, 1, 1, 0);
  gtk_widget_show (scalebox);

  /* Create Brightness Scale */
  frame = gtk_frame_new("Brightness");
  gtk_box_pack_start( GTK_BOX (scalebox), frame, 1, 1, 0);
  gtk_widget_show (frame);

  adjb = gtk_adjustment_new( ib, 0, 255, 1, 10, 1);
  scaleb = gtk_hscale_new(GTK_ADJUSTMENT(adjb));
  gtk_signal_connect (GTK_OBJECT (adjb), "value_changed",
		      GTK_SIGNAL_FUNC (setbrightness), NULL);
  gtk_scale_set_digits(GTK_SCALE(scaleb), 0);
  gtk_container_add (GTK_CONTAINER (frame), scaleb);
  gtk_widget_show (scaleb);
  
  /* Create White Balance Scale */
  frame = gtk_frame_new("White Balance");
  gtk_box_pack_start( GTK_BOX (scalebox), frame, 1, 1, 0);
  gtk_widget_show (frame);
  adjw = gtk_adjustment_new( iw, 0, 255, 1, 10, 1);
  scalew = gtk_hscale_new(GTK_ADJUSTMENT(adjw));
  gtk_signal_connect (GTK_OBJECT (adjw), "value_changed",
		      GTK_SIGNAL_FUNC (setwhite), NULL);
  gtk_scale_set_digits(GTK_SCALE(scalew), 0);
  gtk_container_add (GTK_CONTAINER (frame), scalew);
  gtk_widget_show (scalew);

  /* Create Contrast Scale */
  frame = gtk_frame_new("Contrast");
  gtk_box_pack_start( GTK_BOX (scalebox), frame, 1, 1, 0);
  gtk_widget_show (frame);

  adjc = gtk_adjustment_new( ic, 0, 255, 1, 10, 1);
  scalec = gtk_hscale_new(GTK_ADJUSTMENT(adjc));
  gtk_signal_connect (GTK_OBJECT (adjc), "value_changed",
		      GTK_SIGNAL_FUNC (setcontrast), NULL);
  gtk_scale_set_digits(GTK_SCALE(scalec), 0);
  gtk_container_add (GTK_CONTAINER (frame), scalec);
  gtk_widget_show (scalec);

  /* Create sizedepthbox */
  sizedepthbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start( GTK_BOX (outerbox), sizedepthbox, 1, 1, 0);
  gtk_widget_show (sizedepthbox);

  /* This is the size... */
  frame = gtk_frame_new("Size");
  gtk_box_pack_start( GTK_BOX (sizedepthbox), frame, 1, 1, 0);
  gtk_widget_show (frame);

  innerbox = gtk_vbox_new (TRUE, 0);
  gtk_container_add (GTK_CONTAINER (frame), innerbox);
  gtk_widget_show (innerbox);

  radiobutton = gtk_radio_button_new_with_label (NULL, "Full");
  gtk_box_pack_start (GTK_BOX (innerbox), radiobutton, TRUE, TRUE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
  gtk_signal_connect_object (GTK_OBJECT (radiobutton), "clicked", GTK_SIGNAL_FUNC(setsizefull), NULL);
  gtk_widget_show (radiobutton);
  
  radiogroup = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton));
  radiobutton = gtk_radio_button_new_with_label(radiogroup, "Half");
  gtk_box_pack_start (GTK_BOX (innerbox), radiobutton, TRUE, TRUE, 0);
  gtk_widget_show (radiobutton);
  gtk_signal_connect_object (GTK_OBJECT (radiobutton), "clicked", GTK_SIGNAL_FUNC(setsizehalf), NULL);
  
  radiobutton = gtk_radio_button_new_with_label( gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton)), "Quarter");
  gtk_box_pack_start (GTK_BOX (innerbox), radiobutton, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (radiobutton), "clicked", GTK_SIGNAL_FUNC(setsizequarter), NULL);
  gtk_widget_show (radiobutton);

  /* ...and this is the depth */
  frame = gtk_frame_new("Depth");
  gtk_box_pack_start( GTK_BOX (sizedepthbox), frame, 1, 1, 0);
  gtk_widget_show (frame);

  innerbox = gtk_vbox_new (TRUE, 0);
  gtk_container_add (GTK_CONTAINER (frame), innerbox);
  gtk_widget_show (innerbox);

  radiobutton = gtk_radio_button_new_with_label (NULL, "4 bpp");
  gtk_box_pack_start (GTK_BOX (innerbox), radiobutton, TRUE, TRUE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
  gtk_widget_show (radiobutton);
  
  radiogroup = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton));
  radiobutton = gtk_radio_button_new_with_label(radiogroup, "6 bpp");
  gtk_box_pack_start (GTK_BOX (innerbox), radiobutton, TRUE, TRUE, 0);
  gtk_widget_show (radiobutton);

  /* Create a couple buttons */
  innerbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start( GTK_BOX (topbox), innerbox, 1, 1, 0);
  gtk_widget_show (innerbox);

  button = gtk_button_new_with_label ("Snap Picture");
  gtk_box_pack_start( GTK_BOX (innerbox), button, 1, 1, 0);
  gtk_widget_show(button);

  button = gtk_button_new_with_label ("Freeze Picture");
  gtk_box_pack_start( GTK_BOX (innerbox), button, 1, 1, 0);
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label ("Quit");
  gtk_box_pack_start( GTK_BOX (innerbox), button, 1, 1, 0);
  gtk_widget_show(button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (delete_event), NULL);

  gtk_widget_show (outerbox);
  gtk_widget_show (topbox);
  gtk_widget_show (window);
  
  gtk_timeout_add (1000, clear, (gpointer) drawing_area);

  gtk_main ();
  
  return 0;
}
